@@ -8,6 +8,7 @@
 #include <glib/gstdio.h>
 #include <string.h>
 #include <signal.h>
+#include <math.h>
 #ifdef MAEMO2
   #include <hildon-fm/hildon-widgets/hildon-file-chooser-dialog.h>
   #include <hildon-widgets/hildon-defines.h>
@@ -27,7 +28,7 @@ gint session_last = -1;
 gsize session_number = 0;
 gboolean had_error, paused, channel_closed, first_load_groups = TRUE;
 gchar *argv_session, *error_list;
-gboolean cmdline_session = FALSE, dryrunning = FALSE;
+gboolean cmdline_session = FALSE, cmdline_execute = FALSE, dryrunning = FALSE;
 GtkWidget *main_window, *rsync = NULL;
 gchar config_command[MAXPATH];
 gboolean config_output = FALSE, config_remember = TRUE, config_errorlist = TRUE, config_log = FALSE;
@@ -140,6 +141,7 @@ void load_settings(GtkWidget* main, gchar* session) {
 
 	set_checkbox(main, "check_com_before", g_key_file_get_boolean(settings_file, session, "check_com_before", NULL));
 	set_checkbox(main, "check_com_after", g_key_file_get_boolean(settings_file, session, "check_com_after", NULL));
+	set_checkbox(main, "check_browse_files", g_key_file_get_boolean(settings_file, session, "check_browse_files", NULL));
 
 	g_key_file_free (settings_file);
 }
@@ -202,6 +204,7 @@ void save_settings(GtkWidget* main, gchar *session) {
 
 	g_key_file_set_boolean(settings_file, session, "check_com_before", get_checkbox(main, "check_com_before"));
 	g_key_file_set_boolean(settings_file, session, "check_com_after", get_checkbox(main, "check_com_after"));
+	g_key_file_set_boolean(settings_file, session, "check_browse_files", get_checkbox(main, "check_browse_files"));
 
 	key_data = g_key_file_to_data(settings_file, NULL, &error_handler);
 	key_file = fopen(settings_file_name, "w");
@@ -292,50 +295,62 @@ gboolean load_groups(GtkWidget* main, gchar *session) {
 void show_browse_source(GtkButton *button, gpointer user_data) {
 	GtkWidget *dialog;
 	gint retval;
+	gboolean browse_files = get_checkbox((GtkWidget*) main_window, "check_browse_files");
+	GtkFileChooserAction action = browse_files ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+	gchar *curr_path = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_source"));
+	gchar *curr_folder = g_path_get_dirname(curr_path);
+
 #ifdef MAEMO2
-	dialog = hildon_file_chooser_dialog_new (NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	dialog = hildon_file_chooser_dialog_new (NULL, action);
 #else
-	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, action,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 #endif
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
-		gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_source")));
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), curr_folder);
+	if (browse_files) gtk_file_chooser_select_filename(GTK_FILE_CHOOSER (dialog), curr_path);
 
 	retval = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (retval == GTK_RESPONSE_ACCEPT || retval == GTK_RESPONSE_OK) {
 		gchar *filename, *tmp;
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-		tmp = g_strconcat(filename, "/", NULL);
+		tmp = g_strconcat(filename, browse_files ? "" : "/", NULL);
 		gtk_entry_set_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_source"), tmp);
 		g_free(filename);
 		g_free(tmp);
 	}
 	gtk_widget_destroy (dialog);
+	g_free(curr_folder);
 }
 
 
 void show_browse_dest(GtkButton *button, gpointer user_data) {
 	GtkWidget *dialog;
 	gint retval;
+	gboolean browse_files = get_checkbox((GtkWidget*) main_window, "check_browse_files");
+	GtkFileChooserAction action = browse_files ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+	gchar *curr_path = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_dest"));
+	gchar *curr_folder = g_path_get_dirname(curr_path);
+
 #ifdef MAEMO2
-	dialog = hildon_file_chooser_dialog_new (NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	dialog = hildon_file_chooser_dialog_new (NULL, action);
 #else
-	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, action,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 #endif
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
-		gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_dest")));
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), curr_folder);
+	if (browse_files) gtk_file_chooser_select_filename(GTK_FILE_CHOOSER (dialog), curr_path);
 
 	retval = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (retval == GTK_RESPONSE_ACCEPT || retval == GTK_RESPONSE_OK) {
 		gchar *filename, *tmp;
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
-		tmp = g_strconcat(filename, "/", NULL);
+		tmp = g_strconcat(filename, browse_files ? "" : "/", NULL);
 		gtk_entry_set_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_dest"), tmp);
 		g_free (filename);
 		g_free(tmp);
 	}
 	gtk_widget_destroy (dialog);
+	g_free(curr_folder);
 }
 
 
@@ -485,6 +500,7 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 #endif
 	}
 	g_free(stmp);
+	if (cmdline_execute) on_play_clicked((GtkButton*) window, NULL);
 }
 
 
@@ -508,6 +524,7 @@ void on_close_clicked(GtkButton *button, gpointer user_data) {
 		set_main_title(groups[session_last], NULL);
 		rsync = NULL;
 	}
+	if (cmdline_execute) on_main_destroy((GtkObject*) main_window, NULL);
 }
 
 
@@ -600,6 +617,8 @@ void rsync_cleanup(gpointer data) {
 			g_free(error_list);
 		}
 		dryrunning = FALSE;
+		
+		if (cmdline_execute) on_close_clicked((GtkButton*) data, NULL);
 	} else {
 		channel_closed = TRUE;
 	}
